@@ -1,47 +1,39 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, unicode_literals
+
 from wechatpy.client.api.base import BaseWeChatAPI
 
 
 class WeChatMedia(BaseWeChatAPI):
+    """ 素材管理
+
+    https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html
+    """
 
     def upload(self, media_type, media_file):
         """
-        上传临时素材
+        新增临时素材
         详情请参考
-        http://mp.weixin.qq.com/wiki/5/963fc70b80dc75483a271298a76a8d59.html
+        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/New_temporary_materials.html
 
         :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
         :param media_file: 要上传的文件，一个 File-object
 
         :return: 返回的 JSON 数据包
         """
-        return self._post(
-            url='http://file.api.weixin.qq.com/cgi-bin/media/upload',
-            params={
-                'type': media_type
-            },
-            files={
-                'media': media_file
-            }
-        )
+        return self._post(url='media/upload', params={'type': media_type}, files={'media': media_file})
 
     def download(self, media_id):
         """
         获取临时素材
         详情请参考
-        http://mp.weixin.qq.com/wiki/10/78b15308b053286e2a66b33f0f0f5fb6.html
+        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Get_temporary_materials.html
 
         :param media_id: 媒体文件 ID
 
         :return: requests 的 Response 实例
         """
-        return self._get(
-            'http://file.api.weixin.qq.com/cgi-bin/media/get',
-            params={
-                'media_id': media_id
-            }
-        )
+        return self._get('media/get', params={'media_id': media_id})
 
     def get_url(self, media_id):
         """
@@ -50,14 +42,8 @@ class WeChatMedia(BaseWeChatAPI):
         :param media_id: 媒体文件 ID
         :return: 临时素材下载地址
         """
-        parts = (
-            'http://file.api.weixin.qq.com/cgi-bin/media/get',
-            '?access_token=',
-            self.access_token,
-            '&media_id=',
-            media_id
-        )
-        return ''.join(parts)
+        return 'https://api.weixin.qq.com/cgi-bin/media/get' \
+               '?access_token={}&media_id={}'.format(self.access_token, media_id)
 
     def upload_video(self, media_id, title, description):
         """
@@ -72,7 +58,7 @@ class WeChatMedia(BaseWeChatAPI):
         :return: 返回的 JSON 数据包
         """
         return self._post(
-            url='https://file.api.weixin.qq.com/cgi-bin/media/uploadvideo',
+            url='media/uploadvideo',
             data={
                 'media_id': media_id,
                 'title': title,
@@ -107,19 +93,22 @@ class WeChatMedia(BaseWeChatAPI):
             }
         )
 
-    def upload_mass_image(self, media_file):
+    def upload_image(self, media_file):
         """
         上传群发消息内的图片
         详情请参考
-        http://mp.weixin.qq.com/wiki/15/5380a4e6f02f2ffdc7981a8ed7a40753.html
+        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Adding_Permanent_Assets.html
 
         :param media_file: 要上传的文件，一个 File-object
         :return: 上传成功时返回图片 URL
         """
         res = self._post(
-            url='https://api.weixin.qq.com/cgi-bin/media/uploadimg',
+            url='media/uploadimg',
             files={
                 'media': media_file
-            }
+            },
+            result_processor=lambda x: x['url']
         )
-        return res['url']
+        return res
+
+    upload_mass_image = upload_image
