@@ -1,50 +1,46 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, unicode_literals
-import sys
 import time
 import inspect
+import logging
+import warnings
 
 import six
 import requests
 
+from wechatpy.constants import WeChatErrorCode
+from wechatpy.utils import json, get_querystring
 from wechatpy.session.memorystorage import MemoryStorage
-from wechatpy._compat import json, get_querystring
 from wechatpy.exceptions import WeChatClientException, APILimitedException
 from wechatpy.client.api.base import BaseWeChatAPI
 
 
+logger = logging.getLogger(__name__)
+
+
 def _is_api_endpoint(obj):
     return isinstance(obj, BaseWeChatAPI)
 
 
 class BaseWeChatClient(object):
-
     API_BASE_URL = ''
 
     def __new__(cls, *args, **kwargs):
         self = super(BaseWeChatClient, cls).__new__(cls)
-        if sys.version_info[:2] == (2, 6):
-            import copy
-            # Python 2.6 inspect.gemembers bug workaround
-            # http://bugs.python.org/issue1785
-            for _class in cls.__mro__:
-                if issubclass(_class, BaseWeChatClient):
-                    for name, api in _class.__dict__.items():
-                        if isinstance(api, BaseWeChatAPI):
-                            api = copy.deepcopy(api)
-                            api._client = self
-                            setattr(self, name, api)
-        else:
-            api_endpoints = inspect.getmembers(self, _is_api_endpoint)
-            for name, api in api_endpoints:
-                api_cls = type(api)
-                api = api_cls(self)
-                setattr(self, name, api)
+        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
+        for name, api in api_endpoints:
+            api_cls = type(api)
+            api = api_cls(self)
+            setattr(self, name, api)
         return self
 
-    def __init__(self, access_token=None, session=None):
+    def __init__(self, appid, access_token=None, session=None, timeout=None, auto_retry=True):
+        self._http = requests.Session()
+        self.appid = appid
         self.expires_at = None
         self.session = session or MemoryStorage()
+        self.timeout = timeout
+        self.auto_retry = auto_retry
 
         if isinstance(session, six.string_types):
             from shove import Shove
@@ -57,7 +53,12 @@ class BaseWeChatClient(object):
             storage = ShoveStorage(shove, prefix)
             self.session = storage
 
-        self.session.set('access_token', access_token)
+        if access_token:
+            self.session.set(self.access_token_key, access_token)
+
+    @property
+    def access_token_key(self):
+        return '{0}_access_token'.format(self.appid)
 
     def _request(self, method, url_or_endpoint, **kwargs):
         if not url_or_endpoint.startswith(('http://', 'https://')):
@@ -69,10 +70,6 @@ class BaseWeChatClient(object):
         else:
             url = url_or_endpoint
 
-        # 群发消息上传视频接口地址 HTTPS 证书错误，暂时忽略证书验证
-        if url.startswith('https://file.api.weixin.qq.com'):
-            kwargs['verify'] = False
-
         if 'params' not in kwargs:
             kwargs['params'] = {}
         if isinstance(kwargs['params'], dict) and \
@@ -83,7 +80,9 @@ class BaseWeChatClient(object):
             body = body.encode('utf-8')
             kwargs['data'] = body
 
-        res = requests.request(
+        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
+        result_processor = kwargs.pop('result_processor', None)
+        res = self._http.request(
             method=method,
             url=url,
             **kwargs
@@ -99,37 +98,54 @@ class BaseWeChatClient(object):
                 response=reqe.response
             )
 
-        return self._handle_result(res, method, url, **kwargs)
+        return self._handle_result(
+            res, method, url, result_processor, **kwargs
+        )
 
-    def _handle_result(self, res, method=None, url=None, **kwargs):
-        res.encoding = 'utf-8'
+    def _decode_result(self, res):
         try:
-            result = res.json()
+            result = json.loads(res.content.decode('utf-8', 'ignore'), strict=False)
         except (TypeError, ValueError):
             # Return origin response object if we can not decode it as JSON
+            logger.debug('Can not decode response as JSON', exc_info=True)
             return res
+        return result
+
+    def _handle_result(self, res, method=None, url=None,
+                       result_processor=None, **kwargs):
+        if not isinstance(res, dict):
+            # Dirty hack around asyncio based AsyncWeChatClient
+            result = self._decode_result(res)
+        else:
+            result = res
+
+        if not isinstance(result, dict):
+            return result
 
         if 'base_resp' in result:
             # Different response in device APIs. Fuck tencent!
-            result = result['base_resp']
+            result.update(result.pop('base_resp'))
         if 'errcode' in result:
             result['errcode'] = int(result['errcode'])
 
         if 'errcode' in result and result['errcode'] != 0:
             errcode = result['errcode']
-            errmsg = result['errmsg']
-            if errcode in (40001, 40014, 42001):
-                # access_token expired, fetch a new one and retry request
+            errmsg = result.get('errmsg', errcode)
+            if self.auto_retry and errcode in (
+                    WeChatErrorCode.INVALID_CREDENTIAL.value,
+                    WeChatErrorCode.INVALID_ACCESS_TOKEN.value,
+                    WeChatErrorCode.EXPIRED_ACCESS_TOKEN.value):
+                logger.info('Access token expired, fetch a new one and retry request')
                 self.fetch_access_token()
-                kwargs['params']['access_token'] = self.session.get(
-                    'access_token'
-                )
+                access_token = self.session.get(self.access_token_key)
+                kwargs['params']['access_token'] = access_token
                 return self._request(
                     method=method,
                     url_or_endpoint=url,
+                    result_processor=result_processor,
                     **kwargs
                 )
-            elif errcode == 45009:
+            elif errcode == WeChatErrorCode.OUT_OF_API_FREQ_LIMIT.value:
                 # api freq out of limit
                 raise APILimitedException(
                     errcode,
@@ -147,7 +163,7 @@ class BaseWeChatClient(object):
                     response=res
                 )
 
-        return result
+        return result if not result_processor else result_processor(result)
 
     def get(self, url, **kwargs):
         return self._request(
@@ -156,7 +172,11 @@ class BaseWeChatClient(object):
             **kwargs
         )
 
-    _get = get
+    def _get(self, url, **kwargs):
+        warnings.warn('`_get` method of `WeChatClient` is deprecated, will be removed in 1.6,'
+                      'Use `get` instead',
+                      DeprecationWarning, stacklevel=2)
+        return self.get(url, **kwargs)
 
     def post(self, url, **kwargs):
         return self._request(
@@ -165,11 +185,16 @@ class BaseWeChatClient(object):
             **kwargs
         )
 
-    _post = post
+    def _post(self, url, **kwargs):
+        warnings.warn('`_post` method of `WeChatClient` is deprecated, will be removed in 1.6,'
+                      'Use `post` instead',
+                      DeprecationWarning, stacklevel=2)
+        return self.post(url, **kwargs)
 
     def _fetch_access_token(self, url, params):
         """ The real fetch access token """
-        res = requests.get(
+        logger.info('Fetching access token')
+        res = self._http.get(
             url=url,
             params=params
         )
@@ -196,7 +221,11 @@ class BaseWeChatClient(object):
         expires_in = 7200
         if 'expires_in' in result:
             expires_in = result['expires_in']
-        self.session.set('access_token', result['access_token'], expires_in)
+        self.session.set(
+            self.access_token_key,
+            result['access_token'],
+            expires_in
+        )
         self.expires_at = int(time.time()) + expires_in
         return result
 
@@ -206,7 +235,7 @@ class BaseWeChatClient(object):
     @property
     def access_token(self):
         """ WeChat access token """
-        access_token = self.session.get('access_token')
+        access_token = self.session.get(self.access_token_key)
         if access_token:
             if not self.expires_at:
                 # user provided access_token, just return it
@@ -217,4 +246,4 @@ class BaseWeChatClient(object):
                 return access_token
 
         self.fetch_access_token()
-        return self.session.get('access_token')
+        return self.session.get(self.access_token_key)
